import 'dart:async';

import 'package:flutter/services.dart';
import 'package:vgk_game_services/games_services_platform_interface/game_services_platform_interface.dart';
import 'package:vgk_game_services/games_services_platform_interface/helpers.dart';
import 'package:vgk_game_services/games_services_platform_interface/models/access_point_location.dart';
import 'package:vgk_game_services/games_services_platform_interface/models/achievement.dart';
import 'package:vgk_game_services/games_services_platform_interface/models/result.dart';
import 'package:vgk_game_services/games_services_platform_interface/models/score.dart';

const MethodChannel _channel = MethodChannel("vgk_game_services");

class MethodChannelGamesServices extends GamesServicesPlatform {
  @override
  Future<Result> unlock({achievement = Achievement}) async {
    Result result = Result();
    try {
      var response = await _channel.invokeMethod("unlock", {
        "achievementID": achievement.id,
        "percentComplete": achievement.percentComplete,
      });
      if (response == 'success') {
        result.response = response;
        result.hasError = false;
      } else {
        result.response = response;
        result.hasError = true;
      }
    } catch (e) {
      result.response = e.toString();
      result.hasError = true;
    }
    return result;
  }

  @override
  Future<Result> submitScore({score = Score}) async {
    Result result = Result();
    try {
      var response = await _channel.invokeMethod("submitScore", {
        "leaderboardID": score.leaderboardID,
        "value": score.value,
      });
      if (response == 'success') {
        result.response = response;
        result.hasError = false;
      } else {
        result.response = response;
        result.hasError = true;
      }
    } catch (e) {
      result.response = e.toString();
      result.hasError = true;
    }
    return result;
  }

  @override
  Future<Result> increment({achievement = Achievement}) async {
    Result result = Result();
    try {
      var response = await _channel.invokeMethod("increment", {
        "achievementID": achievement.id,
        "steps": achievement.steps,
      });
      if (response == 'success') {
        result.response = response;
        result.hasError = false;
      } else {
        result.response = response;
        result.hasError = true;
      }
    } catch (e) {
      result.response = e.toString();
      result.hasError = true;
    }
    return result;
  }

  @override
  Future<Result> showAchievements() async {
    Result result = Result();
    try {
      var response = await _channel.invokeMethod("showAchievements");
      if (response == 'success') {
        result.response = response;
        result.hasError = false;
      } else {
        result.response = response;
        result.hasError = true;
      }
    } catch (e) {
      result.response = e.toString();
      result.hasError = true;
    }
    return result;
  }

  @override
  Future<Result> showLeaderboards(
      {iOSLeaderboardID = "", androidLeaderboardID = ""}) async {
    Result result = Result();
    try {
      var response = await _channel.invokeMethod("showLeaderboards", {
        "leaderboardID":
            Helpers.isPlatformAndroid ? androidLeaderboardID : iOSLeaderboardID
      });
      if (response == 'success') {
        result.response = response;
        result.hasError = false;
      } else {
        result.response = response;
        result.hasError = true;
      }
    } catch (e) {
      result.response = e.toString();
      result.hasError = true;
    }
    return result;
  }

  @override
  Future<Result> signIn() async {
    Result result = Result();
    try {
      if (Helpers.isPlatformAndroid) {
        var response = await _channel.invokeMethod("silentSignIn");
        if (response == 'success') {
          result.response = response;
          result.hasError = false;
        } else {
          result.response = response;
          result.hasError = true;
        }
      } else {
        var response = await _channel.invokeMethod("signIn");
        if (response == 'success') {
          result.response = response;
          result.hasError = false;
        } else {
          result.response = response;
          result.hasError = true;
        }
      }
    } catch (e) {
      result.response = e.toString();
      result.hasError = true;
    }
    return result;
  }

  @override
  Future<bool?> get isSignedIn => _channel.invokeMethod("isSignedIn");

  @override
  Future<Result> signOut() async {
    Result result = Result();
    try {
      var response = await _channel.invokeMethod("signOut");
      if (response == 'success') {
        result.response = response;
        result.hasError = false;
      } else {
        result.response = response;
        result.hasError = true;
      }
    } catch (e) {
      result.response = e.toString();
      result.hasError = true;
    }
    return result;
  }

  @override
  Future<Result> showAccessPoint(AccessPointLocation location) async {
    Result result = Result();
    try {
      var response = await _channel.invokeMethod(
          "showAccessPoint", {"location": location.toString().split(".").last});
      if (response == 'success') {
        result.response = response;
        result.hasError = false;
      } else {
        result.response = response;
        result.hasError = true;
      }
    } catch (e) {
      result.response = e.toString();
      result.hasError = true;
    }
    return result;
  }

  @override
  Future<Result> hideAccessPoint() async {
    Result result = Result();
    try {
      var response = await _channel.invokeMethod("hideAccessPoint");
      if (response == 'success') {
        result.response = response;
        result.hasError = false;
      } else {
        result.response = response;
        result.hasError = true;
      }
    } catch (e) {
      result.response = e.toString();
      result.hasError = true;
    }
    return result;
  }

  @override
  Future<Result> getPlayer() async {
    Result result = Result();
    try {
      var response = await _channel.invokeMethod("getPlayer");
      if (response != null && response['errorCode'] == null) {
        result.response = response;
        result.hasError = false;
      } else {
        result.response = response;
        result.hasError = true;
      }
    } catch (e) {
      result.response = e.toString();
      result.hasError = true;
    }
    return result;
  }

  @override
  Future<Result> loadSnapshot({gameName = ""}) async {
    Result result = Result();
    try {
      var response =
          await _channel.invokeMethod("loadSnapshot", {"gameName": gameName});
      if (response != null && response['errorCode'] == null) {
        result.response = response;
        result.hasError = false;
      } else {
        result.response = response;
        result.hasError = true;
      }
    } catch (e) {
      result.response = e.toString();
      result.hasError = true;
    }
    return result;
  }

  @override
  Future<Result> saveSnapshot(
      {gameName = "", dataToSave = "", description = ""}) async {
    Result result = Result();
    try {
      var response = await _channel.invokeMethod("saveSnapshot", {
        "gameName": gameName,
        "dataToSave": dataToSave,
        "description": description,
      });
      if (response != null && response['errorCode'] == null) {
        result.response = response;
        result.hasError = false;
      } else {
        result.response = response;
        result.hasError = true;
      }
    } catch (e) {
      result.response = e.toString();
      result.hasError = true;
    }
    return result;
  }
}
